@@ -1,17 +1,21 @@
 from functools import singledispatch
 from typing import Callable, Iterable, Union, Tuple, TypeVar
 
+from sudachipy.morpheme import Morpheme
+
 from spacy.lang.ja import DetailedToken
 from spacy.language import Language
 from spacy.tokens import Doc, Span, Token
 
 from .bunsetu_recognizer import *
 from .compound_splitter import *
+from .disable_sentencizer import *
 from .ene_ontonotes_mapper import ENE_ONTONOTES_MAPPING
 
 
 __all__ = [
-    "set_split_mode",
+    "make_compound_splitter", "make_bunsetu_recognizer", "make_disable_sentencizer",
+    "force_using_normalized_form_as_lemma", "set_split_mode",
     "token_i", "text", "text_with_ws", "orth", "orth_",
     "ent_type", "ent_type_", "ent_iob", "ent_iob_",
     "lemma", "lemma_", "norm", "norm_",
@@ -41,9 +45,73 @@ __all__ = [
 ]
 
 
+@Language.factory(
+    "compound_splitter",
+    requires=[],
+    assigns=[],
+    retokenizes=True,
+    default_config={"split_mode": None},
+)
+def make_compound_splitter(
+    nlp: Language,
+    name: str,
+    split_mode: str = None,
+):
+    return CompoundSplitter(
+        nlp.vocab,
+        split_mode,
+    )
+
+
+@Language.factory(
+    "bunsetu_recognizer",
+    requires=["token.dep"],
+    assigns=["token.dep"],
+    retokenizes=False,
+    default_config={},
+)
+def make_bunsetu_recognizer(
+    nlp: Language,
+    name: str,
+    remain_bunsetu_suffix: bool = False,
+):
+    return BunsetuRecognizer(
+        nlp.vocab,
+        remain_bunsetu_suffix,
+    )
+
+@Language.factory(
+    "disable_sentencizer",
+    requires=[],
+    assigns=[],
+    retokenizes=False,
+    default_config={},
+)
+def make_disable_sentencizer(
+    nlp: Language,
+    name: str,
+):
+    return DisableSentencizer(
+        nlp.vocab,
+    )
+
+
+_morpheme_dictionary_form = None
+
+
+def force_using_normalized_form_as_lemma(force: bool):
+    global _morpheme_dictionary_form
+    if force and not _morpheme_dictionary_form:
+        _morpheme_dictionary_form = Morpheme.dictionary_form
+        Morpheme.dictionary_form = Morpheme.normalized_form
+    elif not force and _morpheme_dictionary_form:
+        Morpheme.dictionary_form = _morpheme_dictionary_form
+
+
 def set_split_mode(nlp: Language, mode: str):
-    splitter = nlp.get_pipe("CompoundSplitter")
-    splitter.split_mode = mode
+    if nlp.has_pipe("compound_splitter"):
+        splitter = nlp.get_pipe("compound_splitter")
+        splitter.split_mode = mode
 
 
 # token field getters
@@ -152,15 +220,22 @@ def ent_label_ontonotes(token: Token) -> str:
 
 # token field getters for Doc.user_data
 
-def reading_form(token: Token, use_orth_if_none=True) -> str:
-    reading = token.doc.user_data["reading_forms"][token.i]
-    if not reading and use_orth_if_none:
-        reading = token.orth_
-    return reading
+def reading_form(token: Token, use_orth_if_none: bool) -> str:
+    reading = token.morph.get("Reading")
+    if reading:
+        return reading[0]
+    elif use_orth_if_none:
+        return token.orth_
+    else:
+        return None
 
 
 def inflection(token: Token) -> str:
-    return token.doc.user_data["inflections"][token.i]
+    inf = token.morph.get("Inflection")
+    if inf:
+        return inf[0].replace(";", ",")
+    else:
+        return ""
 
 
 # bunsetu related field getters for Doc.user_data
@@ -380,7 +455,7 @@ def _sub_tokens(
                 token.tag_,
                 inflection(token),
                 token.lemma_,
-                reading_form(token),
+                reading_form(token, True),
                 None,
             )
         ]
